@@ -1,17 +1,9 @@
-#
-# This file is part of HTTP-Tiny
-#
-# This software is copyright (c) 2011 by Christian Hansen.
-#
-# This is free software; you can redistribute it and/or modify it under
-# the same terms as the Perl 5 programming language system itself.
-#
 package t::Util;
 
 use strict;
 use warnings;
 
-use IO::File q[SEEK_SET];
+use IO::File qw(SEEK_SET SEEK_END);
 use IO::Dir;
 
 BEGIN {
@@ -72,12 +64,16 @@ sub dir_list {
 sub slurp (*) {
     my ($fh) = @_;
 
+    seek($fh, 0, SEEK_END)
+      || die(qq/Couldn't navigate to EOF on file handle: '$!'/);
+
+    my $exp = tell($fh);
+
     rewind($fh);
 
     binmode($fh)
       || die(qq/Couldn't binmode file handle: '$!'/);
 
-    my $exp = -s $fh;
     my $buf = do { local $/; <$fh> };
     my $got = length $buf;
 
@@ -110,7 +106,13 @@ sub hashify {
     my %hash;
     for my $line ( @$lines ) {
         my ($k,$v) = ($line =~ m{^([^:]+): (.*)$}g);
-        $hash{$k} = $v;
+        $hash{$k} = [ $hash{$k} ] if exists $hash{$k} && ref $hash{$k} ne 'ARRAY';
+        if ( ref($hash{$k}) eq 'ARRAY' ) {
+            push @{$hash{$k}}, $v;
+        }
+        else {
+            $hash{$k} = $v;
+        }
     }
     return %hash;
 }
@@ -150,9 +152,12 @@ sub sort_headers {
         *HTTP::Tiny::Handle::can_write = sub {1};
         *HTTP::Tiny::Handle::connect = sub {
             my ($self, $scheme, $host, $port) = @_;
-            $self->{host} = $monkey_host = $host;
-            $self->{port} = $monkey_port = $port;
+            $self->{host}   = $monkey_host = $host;
+            $self->{port}   = $monkey_port = $port;
+            $self->{scheme} = $scheme;
             $self->{fh} = shift @req_fh;
+            $self->{pid} = $$;
+            $self->{tid} = HTTP::Tiny::Handle::_get_tid();
             return $self;
         };
         my $original_write_request = \&HTTP::Tiny::Handle::write_request;
@@ -162,6 +167,9 @@ sub sort_headers {
             $self->{fh} = shift @res_fh;
         };
         *HTTP::Tiny::Handle::close = sub { 1 }; # don't close our temps
+
+        # don't try to proxy in mock-mode
+        delete $ENV{$_} for map { $_, uc($_) } qw/http_proxy https_proxy all_proxy/;
     }
 }
 
